
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_STAT_PHASE_RECORD_H
#define PG_STAT_PHASE_RECORD_H

#include "base/StatIntvlRec.h"
#include "base/GoalRec.h"
#include "base/ContTypeStat.h"
#include "base/ConnCloseStat.h"
#include "base/HRHistStat.h"
#include "base/OidGenStat.h"
#include "base/histograms.h"

class StatPhaseMgr;

// accumulates all phase stats that are not handled by StatIntvlRec

class StatPhaseRec: public StatIntvlRec {
	public:
		StatPhaseRec();
	
		const String &name() const { return theName; }

		virtual OLog &store(OLog &log) const;
		virtual ILog &load(ILog &);

		void concat(const StatPhaseRec &p); // sequential phases
		void merge(const StatPhaseRec &p);  // concurrent phases

		ostream &print(ostream &os, const String &pfx) const;

	protected:
		void join(const StatPhaseRec &p);

	public: // read-only
		String theName;
		GoalRec theGoal;  // maintained for us

		ContTypeStat theContType;   // per-content-type stats
		ConnCloseStat theConnClose; // pre-conn-close-class stats

		SizeHist theSockRdSzH;   // socket reads
		SizeHist theSockWrSzH;   // socket writes

		/* HTTP */
		HRHistStat theBasicXacts;
		HRHistStat theImsXacts;
		TmSzHistStat theReloadXacts;

		/* ICP */
		HRHistStat theIcpXacts;

		TimeVsSizeHist theRptmVsSz;
		TimeVsUseHist theConnLifeVsUse;

		OidGenStat theOidGen;
};

#endif
