
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "xstd/rndDistrs.h"
#include "base/RndPermut.h"
#include "pgl/RobotSym.h"
#include "client/PopModel.h"
#include "client/CltCfg.h"
#include "client/cltReqTypes.h"


CltCfg::CltCfg(): theOriginSel(0), theRTypeSel(0), thePopModel(0),
	thePublicRatio(-1), theRecurRatio(-1), theEmbedRecurRatio(-1),
	theWaitXactLmt(-1), theIcpPort(-1),
	theSrvCount(0), genUniqUrls(false) {
}

void CltCfg::configure(const RobotSym *cfg) {
	cfg->waitXactLimit(theWaitXactLmt);
	thePeerHttp = cfg->peerHttp();
	thePeerIcp = cfg->peerIcp();
	cfg->icpPort(theIcpPort);

	theLaunchWin = cfg->launchWin();
	cfg->publicRatio(thePublicRatio);
	cfg->recurRatio(theRecurRatio);
	cfg->embedRecurRatio(theEmbedRecurRatio);
	cfg->uniqueUrls(genUniqUrls);

	if (PopModelSym *pms = cfg->popModel()) {
		thePopModel = new PopModel;
		thePopModel->configure(pms);
	}

	configureServers(cfg);
	configureRTypes(cfg);

	if (!thePopModel && theRecurRatio > 0) {
		cerr << cfg->loc() << "popularity model must be specified for"
			<< " positive recurrence ratio (robot " << cfg->kind() << ')'
			<< endl;
		exit(-2);
	}	
}

void CltCfg::configureServers(const RobotSym *cfg) {
	Array<NetAddr*> addrs;
	if (!cfg->origins(addrs, theOriginSel) || !addrs.count()) {
		cerr << cfg->loc() << "no origin addresses specified for robot " << cfg->kind() << endl;
		exit(-2);
	}
	Assert(theOriginSel);
	theOriginSel->rndGen(GenRndGen("client_origins"));
	theSrvCount = addrs.count();

	// get rid of addresses
	while (addrs.count())
		delete addrs.pop();
}

void CltCfg::configureRTypes(const RobotSym *cfg) {
	theRTypeSel = cfg->reqTypes(CltReqTypeIdx());
	if (!theRTypeSel)
		theRTypeSel = new ConstDistr(new RndGen, rtpBasic); // default
	theRTypeSel->rndGen(GenRndGen("client_req_types"));
}
