
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "xstd/gadgets.h"
#include "runtime/LogComment.h"
#include "runtime/ErrorMgr.h"
#include "runtime/polyErrors.h"
#include "client/NidsStore.h"


int NidsStore::TheFillingStoreCount = 0;
int NidsStore::ThePrefetchLmt = -1;
int NidsStore::TheReqLmt = -1;


NidsStore::NidsStore(): theCount(0), theLastNid(0), theMaxFillLvl(0) {
}

void NidsStore::configure(int prefetchLmt, int reqLmt) {
	ThePrefetchLmt = prefetchLmt;
	TheReqLmt = reqLmt;
	theNids.resize((int)(rint(ThePrefetchLmt/(double)TheReqLmt)));
	if (!wasFull())
		TheFillingStoreCount++;
}

void NidsStore::reset() {
	const bool wasWasFull = wasFull();
	theNids.reset();
	theCount = 0;
	theLastNid = 0;
	theMaxFillLvl = 0;
	if (wasWasFull && !wasFull())
		TheFillingStoreCount++;
}

bool NidsStore::wasFull() const {
	return theMaxFillLvl >= TheReqLmt*theNids.capacity() - TheReqLmt;
}

int NidsStore::shortage() const {
	return TheReqLmt*(theNids.capacity() - theNids.count());
}

int NidsStore::next() {
	Assert(!empty());
	const bool wasWasFull = wasFull();

	theLastNid = theNids.top().next();
	theCount--;
	if (theNids.top().exhausted())
		theNids.dequeue();

	if (wasWasFull && !wasFull())
		TheFillingStoreCount++;

	return theLastNid;
}

void NidsStore::noteNoNids() {
	// report insufficient number of new oids if we had enough 
	// oids at some time
	if (wasFull()) {
		const bool verbose = ReportError(errNoNewOids);
		const int storeLimit = 10*ThePrefetchLmt;
		if (theNids.capacity() < storeLimit) {
			// note: resizing may change the wasFull() value
			theNids.resize(Max(1, 2*theNids.capacity()));
			if (verbose)
				Comment(4) << "grew new oid storage to " << theNids.size() << "bytes" << endc;
			if (theNids.capacity() >= storeLimit)
				ReportError(errNidStoreLimit);
			if (!wasFull())
				TheFillingStoreCount++;
		}
	}
}

void NidsStore::add(int start, int count) {
	Assert(count > 0 && start > 0);

	// ignore packs that lead to overflow conditions
	if (theNids.capacity() < theNids.count() + 1) {
		ReportError(errNidRepFlood);
		return;
	}

	const bool wasWasFull = wasFull();
	theNids.enqueue(NidPack(start, start + count));
	theCount += count;
	if (theMaxFillLvl < theCount)
		theMaxFillLvl = theCount;

	if (!wasWasFull && wasFull()) {
		TheFillingStoreCount--;
		if (!TheFillingStoreCount)
			Comment(6) << "fyi: new oid storage is primed" << endc;
	}

}
