
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "xstd/gadgets.h"
#include "base/OLog.h"
#include "base/ILog.h"
#include "runtime/httpHdrs.h"
#include "runtime/LogComment.h"
#include "runtime/ErrorMgr.h"
#include "runtime/polyErrors.h"
#include "client/ServerRep.h"


int ServerRep::TheSharedWss = -1;
int ServerRep::TheNidsPrefetchDepth = -1;
int ServerRep::TheNidsPerReqMax = -1;


void ServerRep::ConfigureNids(int prefetchLmt, int perReqMax) {
	TheNidsPrefetchDepth = prefetchLmt;
	TheNidsPerReqMax = perReqMax;
}

ServerRep::ServerRep(const NetAddr &anAddr, int aHostIdx): 
	theAddr(anAddr), theHostIdx(aHostIdx),
	theSecOid(0), theAccCount(0), 
	theLastRepPos(-1), thePendNidPack(1), theWss(-1) {

	theNids.configure(TheNidsPrefetchDepth, TheNidsPerReqMax);
	if (TheNidsPrefetchDepth > -1)
		theFailedOids.resize(TheNidsPrefetchDepth/10);
}

int ServerRep::nidShortage() const {
	//cerr << here << TheNewOidsPerReqMax << ": " << theNewOids.capacity() << " - (" << thePendCnt << " + " << theNewOids.count() << ") = " << shortage << " : " << req_size << endl;
	return Min(TheNidsPerReqMax, theNids.shortage());
}

int ServerRep::newOid() {
	Assert(haveNewOids());
	return theNids.next();
}

int ServerRep::failedOid() {
	Assert(haveFailedOids());
	return theFailedOids.dequeue();
}

void ServerRep::noteFailedOid(int oid) {
	Assert(oid > 0);
	if (oid <= theSecOid) // no need to report
		return;

	if (theFailedOids.full())
		ReportError(errFidFlood);
	else
		theFailedOids.enqueue(oid);
}

void ServerRep::syncSrvInfo(const RepHdr &rep) {
	theAccCount++;
	syncSrvId(rep.theSrvWid);

	if (rep.theSendPos > theLastRepPos) {
		theLastRepPos = rep.theSendPos;

		if (rep.theSecOid > theSecOid)
			theSecOid = rep.theSecOid;

		syncWss(rep.theSrvWss);
		theHotSetPos = rep.theHotSetPos;

		if (rep.theNidCount > 0 && rep.theNidPack >= thePendNidPack) {
			theNids.add(rep.theNidStart, rep.theNidCount);
			thePendNidPack = rep.theNidPack + 1;
		}
	}
}

void ServerRep::freezeWss() {
	// do not allow temporary lockup with theSecOid to affect WS
	syncWss(Max(theSecOid, theNids.last()));
}

void ServerRep::syncSrvId(const UniqId &aSrvId) {
	if (aSrvId) {
		if (!theId)
			theId = aSrvId;
		else
		if (theId != aSrvId) {
			ReportError(errSrvChangedWid);
			theNids.reset();
			theId = aSrvId;
			theSecOid = 0;
			theLastRepPos = -1;
			thePendNidPack = 1;
		}
	}
}

void ServerRep::syncWss(int newWss) {
	if (newWss > theWss) {
		if (TheSharedWss < 0)
			TheSharedWss = newWss;
		else
		if (theWss >= 0)
			TheSharedWss -= theWss;
		TheSharedWss += newWss;

		theWss = newWss;
	}
}

void ServerRep::store(OLog &log) const {
	log
		<< theAddr
		<< theId
		<< theSecOid
		<< theWss
		<< theAccCount
		;
}

void ServerRep::load(ILog &log) {
	log
		>> theAddr
		>> theId
		>> theSecOid
		>> theWss
		>> theAccCount
		;
}
