
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

// content database "driver"


#include "polygraph.h"

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>

#include "ContentDbase.h"
#include "cdbEntries.h"
#include "base/BStream.h"

inline ostream &xexit(ostream &os) { exit(-2); return os; }


class HtmlParser {
	public:
		HtmlParser();

		void configure(ContentDbase *aDb);
		bool parse(istream *is, const String &fname);

		bool parse();

	protected:
		void scan();
		void nextStr(ostrstream &os, char eos);
		bool nextChar();

	protected:
		ContentDbase *theDb;
		String theFName;
		istream *theStream;

		char theNextChar;
};


static String ThePrgName = "";


HtmlParser::HtmlParser(): theDb(0), theStream(0) {
}

void HtmlParser::configure(ContentDbase *aDb) {
	Assert(!theDb && aDb);
	theDb = aDb;
}

bool HtmlParser::parse(istream *is, const String &fname) {
	Assert(theDb);
	Assert(is);

	theStream = is;
	theFName = fname;

	if (nextChar())
		scan();

	return !theStream->bad();
}

void HtmlParser::scan() {

	do {

		ostrstream buf;
		CdbEntry *e = 0;

		if (theNextChar == '<') {
			e = new CdbeTag;
			nextStr(buf, '>');
			nextChar();
		} else {
			e = new CdbeText;
			buf << theNextChar;
			nextStr(buf, '<');
		}

		Assert(e);
		char *str = buf.str();
		// change absolute URLs to relative ones
		char *abs_url = str;
		while ((abs_url = strstr(abs_url, "://")))
			memcpy(abs_url, "_-2", 3);
		e->image(str);
		delete[] str;
		theDb->add(e);

	} while (*theStream);
}

void HtmlParser::nextStr(ostrstream &os, char eos) {
	char c;
	while (theStream->get(c) && (c != eos))
		os << c;
	os << ends;

	if (c == eos)
		theNextChar = eos;
}

bool HtmlParser::nextChar() {
	if (*theStream)
		return theStream->get(theNextChar);
	else
		return false;
}


static
bool readCont(ContentDbase *cdb, const String &fname) {
	const String hname = fname == "-" ? String("stdin") : fname;
	clog << ThePrgName << ": adding objects from " << hname << " ... ";

	const int savedCount = cdb->count();
	HtmlParser p;
	p.configure(cdb);

	istream *is = fname == "-" ? (istream*)&cin : (istream*)new ifstream (fname);
	const bool res = p.parse(is, hname);
	if (is != &cin)
		delete is;

	if (res)
		clog << "\t " << setw(6) << (cdb->count() - savedCount) << " objects";
	clog << endl;
	return res;
}

static
int doShow(const String &dbName) {
	ContentDbase *cdb = new ContentDbase;
	ifstream f(dbName);
	IBStream is;
	is.configure(&f, dbName);
	cdb->load(is);
	if (is) {
		cdb->print(cout);
		return 0;
	}

	cerr << ThePrgName << ": error loading db from `" << dbName << "'" << endl;
	return -2;
}

static
int doAdd(const String &dbName, int fcount, char *fnames[]) {
	ContentDbase *cdb = new ContentDbase;
	fstream f(dbName, ios::in|ios::out);

	{
		clog << ThePrgName << ": loading db from " << dbName << " ... ";
		IBStream is;
		is.configure(&f, dbName);
		cdb->load(is);
		clog << "\t " << setw(6) << cdb->count() << " objects" << endl;
	}

	if (fcount) {
		for (int i = 0; i < fcount; ++i)
			Must(readCont(cdb, fnames[i]));
	} else {
		Must(readCont(cdb, "-"));
	}

	{
		clog << ThePrgName << ": storing db in " << dbName << " ... ";
		f.clear();
		f.seekg(0);
		OBStream os;
		os.configure(&f, dbName);
		cdb->store(os);
		if (os)
			clog << "\t " << setw(6) << cdb->count() << " objects" << endl;
		else
			clog << "error: " << error(errno) << endl;
	}
	f.close();

	return 0;
}

static
int usage() {
	cerr << "usage: " << ThePrgName << " <database.cdb> <command> [file.html ...]" << endl
		<< "commands: " << endl
		<< "\tshow  - " << " dump db contents to stdout" << endl
		<< "\tadd   - " << " absorb file(s) contents" << endl;
	return 0;
}

int main(int argc, char *argv[]) {
	ThePrgName = argv[0];

	if (argc < 3)
		return usage();

	const String dbName = argv[1];
	const String cmd = argv[2];

	if (cmd == "show")
		return doShow(dbName);
	else
	if (cmd == "add")
		return doAdd(dbName, argc-3, argv+3);


	usage();
	cerr << "unknown command `" << cmd << "', expected `show' or `add'" << endl;
	return -1;
}
