
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <iomanip.h>

#include "base/RndPermut.h"
#include "base/ObjId.h"
#include "csm/ContentCfg.h"
#include "csm/ContentMgr.h"
#include "runtime/HostMap.h"
#include "oid2Url.h"


static
void Oid2UrlName(const ObjId &oid, ostream &os) {
	const ios::fmtflags flags = os.flags();
	const char fill = os.fill('0');
	os << hex << setfill('0');
	os << 't' << setw(2) << oid.type() << '/';
	os << '_' << setw(8) << oid.name();
	os.fill(fill);
	os.flags(flags);

	if (oid.type() > 0) {
		const ContentCfg *ccfg = TheContentMgr.get(oid.type());
		Assert(ccfg);
		const int seed = RndPermut(oid.hash(), rndContentExt);
		if (const String &ext = ccfg->ext(seed))
			os << ext;
	}
}

void Oid2UrlPath(const ObjId &oid, ostream &os) {
	os << 'w' << oid.world() << '/';
	Oid2UrlName(oid, os);
}

void Oid2Url(const ObjId &oid, ostream &os) {
	Assert(TheHostMap);
	os << "http://";
	if (const HostCfg *cfg = TheHostMap->at(oid.server())) {
		if (cfg->theAddr.port() == 80)
			os << cfg->theAddr.addrA();
		else
			os << cfg->theAddr;
	} else {
		os << oid.server();
	}
	os << '/';

	Oid2UrlPath(oid, os);
}

