
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "PglRec.h"
#include "PglArraySym.h"
#include "PglNetAddrSym.h"
#include "HostsBasedSym.h"


HostsBasedSym::HostsBasedSym(const String &aType): RecSym(aType, new PglRec) {
	theRec->bAdd("addr[]", "hosts", 0);
}

HostsBasedSym::HostsBasedSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool HostsBasedSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "HostsBased";
}

SynSym *HostsBasedSym::clone(const String &type) const {
	if (isA(type))
		return new HostsBasedSym(this->type(), theRec->clone());
	return RecSym::clone(type);
}

int HostsBasedSym::hostCount() const {
	if (ArraySym *a = getArraySym("hosts"))
		return a->count();
	else
		return 0;
}

NetAddrSym *HostsBasedSym::host(int idx) const {
	if (ArraySym *a = getArraySym("hosts")) {
		Assert(idx < a->count());
		return &(NetAddrSym&)(*a)[idx]->cast("addr");
	}
	return 0;
}

bool HostsBasedSym::hosts(Array<NetAddrSym*> &addrs) const {
	if (ArraySym *a = getArraySym("hosts")) {
		ArraySymExport(*a, "addr", addrs);
		return true;
	}
	return false;
}
