
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl.h"

#include "PglBoolSym.h"


BoolSym::BoolSym(bool aVal = -1): ExpressionSym("bool"), theVal(aVal) {
}

bool BoolSym::isA(const String &type) const {
	return ExpressionSym::isA(type) ||
		type == "Bool" || type == "bool";
}

SynSym *BoolSym::clone(const String &type) const {
	if (isA(type))
		return new BoolSym(theVal);
	return ExpressionSym::clone(type);
}

ExpressionSym *BoolSym::unOper(const String &op) const {
	if (op == "!")
		return new BoolSym(!theVal);
	else
		return ExpressionSym::unOper(op);
}


ExpressionSym *BoolSym::bnOper(const String &op, const SynSym &s) const {
	if (op.len() < 1 || op.len() > 2 || !(op == "^" || op[0] == op[1]))
		return ExpressionSym::bnOper(op, s);

	bool otherVal = false;

	if (s.isA("bool")) {
		otherVal = ((const BoolSym&)s.cast("bool")).val();
	} else 
	if (BoolSym *is = (BoolSym*)s.clone("bool")) {
		otherVal = is->val();
		delete is;
	} else
		return ExpressionSym::bnOper(op, s);

	switch (op[0]) {
		case '|': return new BoolSym(theVal || otherVal);
		case '&': return new BoolSym(theVal && otherVal);
		case '^': return new BoolSym(theVal ^ otherVal);
		default: return ExpressionSym::bnOper(op, s);
	}
}

ostream &BoolSym::print(ostream &os, const String &) const {
	return os << (theVal ? "true" : "false");
}
