
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl.h"

#include "PglRec.h"
#include "ProxySym.h"


ProxySym::ProxySym(): AgentSym("Proxy") {
	theRec->bAdd("Robot", "client", 0);
	theRec->bAdd("Server", "server", 0);
	theRec->bAdd("Cache", "cache", 0);
}

ProxySym::ProxySym(const String &aType, PglRec *aRec): AgentSym(aType, aRec) {
}

bool ProxySym::isA(const String &type) const {
	return AgentSym::isA(type) || type == "Proxy";
}

SynSym *ProxySym::clone(const String &type) const {
	if (isA(type))
		return new ProxySym(this->type(), theRec->clone());
	return AgentSym::clone(type);
}

RobotSym *ProxySym::client() const {
	SynSymTblItem *ri = 0;
	Assert(theRec->find("client", ri));
	Assert(ri->sym());
	return &(RobotSym&)ri->sym()->cast("Robot");
}

ServerSym *ProxySym::server() const {
	SynSymTblItem *si = 0;
	Assert(theRec->find("server", si));
	Assert(si->sym());
	return &(ServerSym&)si->sym()->cast("Server");
}

CacheSym *ProxySym::cache() const {
	SynSymTblItem *ci = 0;
	Assert(theRec->find("cache", ci));
	if (ci->sym())
		return &(CacheSym&)ci->sym()->cast("Cache");
	cerr << loc() << "must configure cache for each proxy" << endl << xexit;
	return 0;
}

