
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_XACTION_CLASS_H
#define PG_XACTION_CLASS_H

#include "FileScanner.h"
#include "base/UniqId.h"
#include "base/Oid.h"

class IOBuf;
class Connection;
class Agent;

class Xaction: public FileScanUser {
	public:
		static int TheSampleDebt; // log xactions until no debt

	public:
		virtual void reset(); // XXX: split into local/virtual reset

		const UniqId &id() const { return theId; }
		const OidRec &oidRec() const { return theOidRec; }
		Connection *conn() { return theConn; }
		int logCat() const { return theLogCat; }
		bool started() const { return theStartTime > 0; }

		Size repSize() const { return theRepSize; }
		Time queueTime() const { return theEnqueTime > 0 ? theStartTime - theEnqueTime : Time::Sec(0); }
		Time lifeTime() const { return theLifeTime; }
		int httpStatus() const { return theHttpStatus; }

		virtual bool needRetry() const { return false; }

		void oidRec(const OidRec &aRec) { theOidRec = aRec; }

	protected:
		enum State { stNone = 0, stSpaceWaiting, stHdrWaiting, stBodyWaiting, stDone };

	protected:
		virtual Agent *owner() = 0;
		virtual void newState(State aState) { theState = aState; }

		void start(Connection *conn);
		void finish(int errNo);

		void stopStats();
		void logStats();
		virtual void logStats(OLog &ol) const;

		void printMsg(const IOBuf &buf) const;
		void printMsg(const IOBuf &buf, Size maxSize) const;
		void printMsg(const char *buf, Size maxSize) const;

	protected:
		static int TheCount; // to report xaction "position"

	protected:
		Connection *theConn;
		UniqId theId;

		Time theEnqueTime;  // put in a queue (optional)
		Time theStartTime;  // started doing something
		Time theLifeTime;   // [start, stop]; set in stop()
		OidRec theOidRec;   // various details about the transfered object
		Size theRepSize;    // reply size
		int theHttpStatus;
		int theLogCat;      // log entry category

		State theState;
};

#endif
