
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include "Socket.h"
#include "String.h"
#include "AlarmClock.h"
#include "LogComment.h"
#include "PolyOLog.h"
#include "base/Progress.h"
#include "SharedOpts.h"
#include "StatPhaseMgr.h"
#include "shutdown.h"

#include "globals.h"


static String TheReason = 0;
static int TheSignal = -1;

static bool DoShutdown = false; // ShutdownReason() has been called
static bool InShutdown = false; // Shutdown() has started, has not finished
static bool IsClean = false;    // Shutdown() has finished

// check various shutdown conditions
bool ShutdownNow() {
	if (DoShutdown)
		return true; // already decided
	else
	if (Socket::Level() >= Socket::TheMaxLevel) {
		Comment << "active file descriptors: " << Socket::Level() << "; limit: " << Socket::TheMaxLevel << endc;
		ShutdownReason("dangerously low on file descriptors");
	} else
	if (TheSignal >= 0) {
		Comment << "noticed shutdown signal (" << TheSignal << ")" << endc;
		ShutdownReason("got shutdown signal");
	} else
		return false;

	return DoShutdown = true;
}

bool ShutdownRequested() {
	return DoShutdown;
}

void ShutdownSignal(int s) {
	TheSignal = s;
	clog << "got shutdown signal (" << s << ")" << endl;
}

void ShutdownReason(const String &reason) {
	if (reason)
		TheReason = reason;
	DoShutdown = true;
}

void Shutdown() {
	if (IsClean || InShutdown)
		return; // nothing to do; was here already?!

	DoShutdown = InShutdown = true;

	Clock::Update(false);

	Comment(2) << "got " << TheProgress.xacts()
		<< " xactions and " << TheProgress.errs() << " errors" << endc;
	if (TheReason)
		Comment << "shutdown reason: " << TheReason << endc;

	// last minute dumps
	TheStatPhaseMgr.forceFlush();

	TheOLog.close();
	TheSmplOLog->close();

	IsClean = true;
	InShutdown = false;
}
