
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef X_HEAP_H
#define X_HEAP_H

#include "Array.h"

// Heap "tree" adopted with modifications from
// "Algorithms in C", Sedgewick, Addison-Wesley, 1990, ISBN 0-201-51425-7

template <class Item>
class Heap: protected Array<Item> {
	public:
		Heap(int aCapacity = 2): Array<Item>(aCapacity), theCnt(0) { theCount = 1; }

		int count() const { return theCnt; }
		bool empty() const { return theCnt <= 0; }

		Item &top() { return theItems[1]; }
		Item &at(int idx) { return theItems[idx+1]; }

		void add(Item v) { append(v); floatUp(++theCnt); }
		void skip() { theItems[1] = theItems[theCnt--]; theCount--; sinkDown(1); }
		Item shift() { const Item v = theItems[1]; skip(); return v; }

		void resize(int aCap) { Array<Item>::stretch(aCap); }

	protected:
		inline void floatUp(int k);
		inline void sinkDown(int k);

	protected:
		int theCnt;  // theCount-1
};


/* implementaion of in-lined methods */

template <class Item>
inline
void Heap<Item>::floatUp(int k) {
	const Item v = theItems[k];
	for (int p = k/2; p && theItems[p] > v; k = p, p /= 2)
		theItems[k] = theItems[p];
	theItems[k] = v;
}

template <class Item>
inline
void Heap<Item>::sinkDown(int k) {
	const int half = theCnt/2;
	const Item v = theItems[k];
	while (k <= half) {
		// smallest of the (at most) two kids
		int j = k*2;
		if (j < theCnt && theItems[j+1] < theItems[j])
			j++;
		if (v < theItems[j])
			break;
		theItems[k] = theItems[j];
		k = j;
	}
	theItems[k] = v;
}

#endif
